<%@ webhandler language="c#" class="ContextViewer" debug="true" %>

using System;
using System.Web;
using System.Reflection;
using System.Runtime.Remoting.Messaging;
using System.Threading;
using System.Diagnostics;

public class ContextViewer : IHttpAsyncHandler
{
    const string CallContextKey = "AsyncRequestState";
    
    // IHttpHandler
    //
    public bool IsReusable { get { return(true); } }
    
    public void ProcessRequest( HttpContext ctx )
    {
        ctx.Response.Output.Write( "IHttpHandler.ProcessRequest called on thread {0}<br/>",
                                   AppDomain.GetCurrentThreadId() );
        ctx.Response.Output.Write( "IHttpHandler.ProcessRequest HttpContext principal: {0}<br/>",
                                   ctx.User.Identity.Name );
        ctx.Response.Output.Write( "IHttpHandler.ProcessRequest thread principal: {0}<br/>",
                                   Thread.CurrentPrincipal.Identity.Name );
        ctx.Response.Output.Write("IHttpHandler.ProcessRequest started sleeping at {0}<br/>", DateTime.Now);
        Thread.Sleep(5000);
        ctx.Response.Output.Write("IHttpHandler.ProcessRequest finished sleeping at {0}<br/>", DateTime.Now);
    }

    public void OnProcessRequestComplete( IAsyncResult ar )
    {
        AsyncRequestState asyncState = CallContext.GetData(CallContextKey) as AsyncRequestState;
        
        if( asyncState != null )
        {
            asyncState.ctx.Response.Output.Write( "Handler.OnProcessRequestComplete called on thread {0}<br/>",
                                                  AppDomain.GetCurrentThreadId() );

            // This triggers the asp.net plumbing to call our
            // IHttpAsyncHandler.EndProcessRequest method.
            //
            asyncState.cb(asyncState.ar);
        }
    }
    
    // IHttpAsyncHandler
    //
    public IAsyncResult BeginProcessRequest( HttpContext ctx, AsyncCallback cb, object extraData )
    {
        ctx.Response.Output.Write( "IHttpAsyncHandler.BeginProcessRequest called on thread {0}<br/>",
                                   AppDomain.GetCurrentThreadId() );

        BreakIfRequested();
        
        AsyncHandlerDelegate asyncHandler = new AsyncHandlerDelegate(ProcessRequest);
        AsyncRequestState asyncState = new AsyncRequestState(ctx, cb, extraData);
        CallContext.SetData(CallContextKey, asyncState);
        asyncState.ar = asyncHandler.BeginInvoke(ctx, new AsyncCallback(OnProcessRequestComplete), null);
        return(asyncState.ar);
    }
    
    public void EndProcessRequest( IAsyncResult ar )
    {
        AsyncRequestState asyncState = CallContext.GetData(CallContextKey) as AsyncRequestState;
        
        if( asyncState != null )
        {
            asyncState.ctx.Response.Output.Write( "IHttpAsyncHandler.EndProcessRequest called on thread {0}<br/>",
                                                  AppDomain.GetCurrentThreadId() );
        }

        // The following call doesn't really do anything, since we don't
        // have any output arguments or return values from our method
        // (other than exceptions).
        //
        ((AsyncHandlerDelegate)(((AsyncResult)ar).AsyncDelegate)).EndInvoke(ar);
        
        if( asyncState != null )
        {
            CallContext.SetData(CallContextKey, null);
        }
    }
    
    delegate void AsyncHandlerDelegate( HttpContext ctx );
    
    class AsyncRequestState : ILogicalThreadAffinative
    {
        public AsyncRequestState( HttpContext ctx, AsyncCallback cb, object extraData )
        {
            this.ctx = ctx;
            this.cb = cb;
            this.extraData = extraData;
        }
        
        public IAsyncResult ar = null;
        public HttpContext ctx;
        public AsyncCallback cb;
        public object extraData;
    }
    
    void BreakIfRequested()
    {
        string breakArg = HttpContext.Current.Request.QueryString["break"];
        
        if( (breakArg != null) && ((breakArg == "yes") || (breakArg == "true")) )
        {
            if( Debugger.IsAttached )
            {
                Debugger.Break();
            }
            else
            {
                Debugger.Launch();
            }
        }
    }
}
